<?php
/**
 * Author: wbolt team
 * Author URI: https://www.wbolt.com
 */

class WB_MagicPost_Move
{

    public function __construct()
    {

        if (is_admin()) {
            add_action('wp_ajax_magicpost',array($this, 'magicpost_ajax'));
        }
    }

    public function magicpost_ajax()
    {
        global $wpdb,$wp_taxonomies,$wp_post_types;

        $op = trim(sanitize_text_field($_POST['op'] ?? ''));

        switch ($op) {
            case 'move_cnf':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $post_types = [];
                    if($wp_post_types && is_array($wp_post_types))foreach($wp_post_types as $type) {
                        if(in_array($type->name,['attachment'])){
                            continue;
                        }
                        if ($type->public) {
                            $post_types[$type->name] = $type->labels->name;
                        }
                    }
                    $cat_tax = [];
                    $cat_type = [];
                    foreach($post_types as $t=>$n){
                        $tax = get_object_taxonomies([$t]);
                        foreach($tax as $tx){
                            $o = get_taxonomy($tx);
                            if(!$o){
                                continue;
                            }
                            if(preg_match('#categor#',$o->meta_box_cb)){
                                $cat_tax[$t] = $tx;
                                $cat_type[$t] = $n;
                                break;
                            }
                        }
                    }

                    $cat_list = [];
                    $cat_table = [];

                    $tree = [];
                    foreach ($cat_tax as $post=>$tax){
                        $cat_list[$post] = $this->get_category($tax);
                        $cat_table[$post] = $this->category_table($cat_list[$post],0);
                        //$tree[$post] = wp_dropdown_categories( ['echo'=>0,'taxonomy'=>$tax,'hierarchical'=>1] );
                    }

                    $ret['cnf'] = [
                        'post_type'=> $cat_type ? $cat_type : ['post'=>'文章'],
                        'cat_tax' => $cat_tax,
                        //'tree' => $tree,
                        'cat_list' => $cat_list,
                        'cat_table' => $cat_table,
                    ];



                } while (0);
                header('content-type:text/json;charset=utf-8;');
                echo json_encode($ret);

                exit();
                break;

            case 'move_query':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $q = $this->sanitize_text($_POST['q'] ?? []);
                    $post_type = trim($q['post_type'] ?? 'post');
                    if(!$post_type){
                        $post_type = 'post';
                    }
                    $cat = intval($q['cat'] ?? 0);
                    $keyword = trim($q['keyword'] ?? '');
                    $tax = trim(sanitize_text_field($_POST['tax'] ?? 'category'));

                    $where = array(
                        $wpdb->prepare("a.post_type = %s",$post_type),
                        "a.post_status = 'publish'"
                    );
                    if($cat){
                        $cat_id = [];
                        $child = get_term_children($cat,$tax);
                        if($child){
                            $cat_id = $child;
                        }
                        array_push($cat_id,$cat);
                        $term_id = implode(',',wp_parse_id_list($cat_id));
                        $where[] = " EXISTS( 
                        SELECT t3.object_id FROM $wpdb->term_taxonomy t2,$wpdb->term_relationships t3
                         WHERE t2.term_id IN($term_id) AND t2.term_taxonomy_id=t3.term_taxonomy_id AND t3.object_id= a.ID
                         )";
                    }

                    if($keyword){
                        $where[] = $wpdb->prepare('a.post_title LIKE %s','%'.$keyword.'%');
                    }


                    $num = 10;
                    if(isset($_POST['num'])){
                        $num = max(10,absint($_POST['num']));
                    }
                    if(isset($_POST['page'])){
                        $page = absint($_POST['page']);
                    }

                    $offset = max(0,($page-1) * $num);

                    if($where){
                        $where = implode(' AND ',$where);
                    }else{
                        $where = '1=1';
                    }


                    $sql = "SELECT SQL_CALC_FOUND_ROWS a.ID,a.post_title,a.post_type,a.post_status,a.post_date 
                        FROM $wpdb->posts a WHERE $where ORDER BY a.post_date ASC LIMIT  $offset,$num";//
                    $list = $wpdb->get_results($sql);
                    $total = $wpdb->get_var("SELECT FOUND_ROWS()");
                    foreach($list as $r){
                        $cat = [];
                        if($tax){
                            $cat_list = wp_get_object_terms( $r->ID, $tax, [] );
                            if($cat_list)foreach ($cat_list as $c){
                                $cat[] = $c->name;
                            }
                        }
                        $r->post_category = $cat;
                    }


                    $ret = array(
                        'q' => $q,
                        'sql'=>$sql,
                        'tx'=>$tax,
                        'num'=>$num,
                        'total'=>$total,
                        'code'=>0,
                        'data'=>$list,
                    );






                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
                break;
            case 'move_cat':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    if(!get_option('wb_magicpost_ver',0)){
                        break;
                    }
                    $post_id = sanitize_text_field($_POST['post'] ?? '');
                    $cat = sanitize_text_field($_POST['cat'] ?? '');
                    $tax = sanitize_text_field($_POST['tax'] ?? 'category');
                    $type = intval($_POST['type'] ?? '');
                    if(!$tax || !$cat || !$post_id || !$type){
                        break;
                    }
                    $id_list = wp_parse_id_list(explode(',',$post_id));
                    $category = wp_parse_id_list(explode(',',$cat));

                    if($type == 1){//重置文章分类为目标分类
                        foreach($id_list as $ID){
                            wp_set_post_terms( $ID, $category, $tax, false );
                        }
                    }else if($type == 2){//添加目标分类至文章
                        foreach($id_list as $ID){
                            wp_set_post_terms( $ID, $category, $tax, true );
                        }
                    }else if($type == 3){//将文章从目标分类移出
                        foreach($id_list as $ID){
                            wp_remove_object_terms($ID,$category,$tax);
                        }
                    }

                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
                break;
            case 'move_tag':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    if(!get_option('wb_magicpost_ver',0)){
                        break;
                    }
                    $source = sanitize_text_field($_POST['source'] ?? '');
                    $target = sanitize_text_field($_POST['target'] ?? '');
                    $tax = sanitize_text_field($_POST['tax'] ?? 'post_tag');
                    $post_type = sanitize_text_field($_POST['post_type'] ?? 'post');
                    $type = intval($_POST['type'] ?? '');
                    if(!$tax || !$source || !$target){
                        break;
                    }
                    $source_list = wp_parse_id_list(explode(',',$source));
                    $target_list = wp_parse_id_list(explode(',',$target));
                    if($type == 1){//迁移原标签文章至目标标签
                        $list = query_posts(['post_type'=>$post_type,'tag__in'=>$source_list]);
                        foreach($list as $r){
                            wp_set_post_terms( $r->ID, $target_list, $tax, true );
                            wp_remove_object_terms($r->ID,$source_list,$tax);
                        }
                    }else if($type == 2){//原标签文章增加目标标签
                        $list = query_posts(['post_type'=>$post_type,'tag__in'=>$source_list]);
                        foreach($list as $r){
                            wp_set_post_terms( $r->ID, $target_list, $tax, true );
                        }
                    }else if($type == 3){//移除文章中的目标标签
                        foreach($source_list as $ID){
                            if(!$ID)continue;
                            wp_remove_object_terms($ID,$target_list,$tax);
                        }
                    }else if($type == 4){//新增目标标签至文章
                        foreach($source_list as $ID){
                            if(!$ID)continue;
                            wp_set_post_terms( $ID, $target_list, $tax, true );
                        }
                    }


                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
            case 'move_del_tag':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $term_id = intval($_POST['id'] ?? 0);
                    $tax = sanitize_text_field($_POST['tax'] ?? 'post_tag');
                    if(!$tax || !$term_id){
                        break;
                    }
                    wp_delete_term($term_id,$tax);

                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();

                break;
            case 'move_update_tag':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $term_id = intval($_POST['id'] ?? 0);
                    $name = trim(sanitize_text_field($_POST['name'] ?? ''));
                    $slug = trim(sanitize_text_field($_POST['slug'] ?? ''));
                    $tax = sanitize_text_field($_POST['tax'] ?? 'post_tag');
                    if(!$tax || !$term_id || !$name || !$slug){
                        break;
                    }
                    $result = wp_update_term($term_id,$tax,['name'=>$name,'slug'=>$slug]);
                    if(is_wp_error($result)){
                        $ret['code'] = 1;
                        $ret['desc'] = $result->get_error_message();
                        break;
                    }

                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
                break;
            case 'move_new_tag':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $name = trim(sanitize_text_field($_POST['name'] ?? ''));
                    $slug = trim(sanitize_text_field($_POST['slug'] ?? ''));
                    $tax = sanitize_text_field($_POST['tax'] ?? 'post_tag');
                    if(!$tax || !$name){
                        $ret['code'] = 1;
                        $ret['desc'] = '参数不能为空';
                        break;
                    }
                    $result = wp_insert_term($name,$tax,['slug'=>$slug]);
                    if(is_wp_error($result)){
                        $ret['code'] = 1;
                        $ret['desc'] = $result->get_error_message();
                        break;
                    }

                    $ret['data'] = $result;
                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
                break;

            case 'move_tag_cnf':
                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $post_types = [];
                    if($wp_post_types && is_array($wp_post_types))foreach($wp_post_types as $type) {
                        if(in_array($type->name,['attachment'])){
                            continue;
                        }
                        if ($type->public) {
                            $post_types[$type->name] = $type->labels->name;
                        }
                    }
                    $cat_tax = [];
                    $cat_type = [];
                    foreach($post_types as $t=>$n){
                        $tax = get_object_taxonomies([$t]);
                        foreach($tax as $tx){
                            $o = get_taxonomy($tx);
                            if(!$o){
                                continue;
                            }
                            //print_r($o->meta_box_cb);
                            if(preg_match('#tags#',$o->meta_box_cb)){
                                $cat_tax[$t] = $tx;
                                $cat_type[$t] = $n;
                                break;
                            }
                        }
                    }


                    $ret['cnf'] = [
                        'post_type'=> $cat_type ? $cat_type : ['post'=>'文章'],
                        'cat_tax' => $cat_tax,
                    ];

                }while(0);
                header('content-type:text/json;charset=utf-8;');
                echo json_encode($ret);

                exit();
                break;

            case 'move_tag_query':

                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $pos = sanitize_text_field($_POST['pos'] ?? '');
                    $q = $this->sanitize_text($_POST['q'] ?? []);
                    $post_type = trim($q['post_type'] ?? 'post');

                    $keyword = trim($q['keyword'] ?? '');
                    $tag_id = intval($q['tag_id'] ?? 0);
                    $tax = trim(sanitize_text_field($_POST['tax'] ?? 'post_tag'));

                    $param = ['taxonomy'=>$tax,'hide_empty'=>false,'pad_counts'=>true,'suppress_filter'=>true];

                    if($tag_id){
                        $param['include'] = $tag_id;
                    }
                    if($keyword){
                        $param['search'] = $keyword;
                    }

                    $num = 10;
                    if(isset($_POST['num'])){
                        $num = max(10,absint($_POST['num']));
                    }
                    if(isset($_POST['page'])){
                        $page = absint($_POST['page']);
                    }

                    $offset = max(0,($page-1) * $num);


                    $param['number'] = $num;
                    $param['offset'] = $offset;



                    $list = get_terms($param);
                    $total = intval($_POST['total'] ?? 0);
                    if($page<2){
                        $total = wp_count_terms($param);
                    }



                    $ret = array(
                        'param'=>$param,
                        'q' => $q,
                        'tx'=>$tax,
                        'num'=>$num,
                        'total'=>$total,
                        'code'=>0,
                        'data'=>$list,
                    );






                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
                break;
            case 'move_tag_post_query':

                $ret = array('code' => 0, 'desc' => 'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $pos = sanitize_text_field($_POST['pos'] ?? '');
                    $q = $this->sanitize_text($_POST['q'] ?? []);
                    $post_type = trim($q['post_type'] ?? 'post');

                    $keyword = trim($q['keyword'] ?? '');
                    $tag = trim($q['tag'] ?? '');
                    $tag_id = intval($q['tag_id'] ?? 0);
                    $tax = trim(sanitize_text_field($_POST['tax'] ?? 'post_tag'));

                    $where = array(
                        $wpdb->prepare("a.post_type = %s",$post_type),
                        "a.post_status = 'publish'"
                    );

                    if($tag_id){
                        $where[] = $wpdb->prepare(" EXISTS( 
                        SELECT t3.object_id FROM $wpdb->term_taxonomy t2,$wpdb->term_relationships t3
                         WHERE t2.term_id = $tag_id AND t2.term_taxonomy_id=t3.term_taxonomy_id AND t3.object_id= a.ID
                        AND t2.taxonomy = %s )",$tax);
                    }

                    if($tag){
                        $where[] = $wpdb->prepare(" EXISTS( 
                        SELECT t3.object_id FROM $wpdb->term_taxonomy t2,$wpdb->term_relationships t3,$wpdb->terms t1
                         WHERE t2.term_taxonomy_id=t3.term_taxonomy_id AND t3.object_id= a.ID AND t2.term_id=t1.term_id
                        AND t2.taxonomy = %s AND CONCAT_WS('',t1.name,t1.slug) LIKE %s  )",$tax,'%'.$tag.'%');
                    }


                    if($keyword){
                        $where[] = $wpdb->prepare('a.post_title LIKE %s','%'.$keyword.'%');
                    }

                    $num = 10;
                    if(isset($_POST['num'])){
                        $num = max(10,absint($_POST['num']));
                    }
                    if(isset($_POST['page'])){
                        $page = absint($_POST['page']);
                    }

                    $offset = max(0,($page-1) * $num);

                    if($where){
                        $where = implode(' AND ',$where);
                    }else{
                        $where = '1=1';
                    }

                    $sql = "SELECT SQL_CALC_FOUND_ROWS a.ID,a.post_title,a.post_type,a.post_status,a.post_date 
                        FROM $wpdb->posts a WHERE $where ORDER BY a.post_date ASC LIMIT  $offset,$num";//
                    $list = $wpdb->get_results($sql);
                    $total = $wpdb->get_var("SELECT FOUND_ROWS()");
                    foreach($list as $r){
                        $cat = [];
                        if($tax){
                            $cat_list = wp_get_object_terms( $r->ID, $tax, [] );
                            if($cat_list)foreach ($cat_list as $c){
                                $cat[] = $c->name;
                            }
                        }
                        $r->post_category = $cat;
                    }

                    $ret = array(
                        'q' => $q,
                        'tx'=>$tax,
                        'num'=>$num,
                        'total'=>$total,
                        'code'=>0,
                        'data'=>$list,
                    );

                } while (0);
                header('content-type:text/json;charset=utf-8;');

                echo json_encode($ret);

                exit();
                break;
        }

    }

    public function category_table($list,$lv){
        $new_list = [];
        foreach($list as $r){
            $r->lv = $lv;
            $new_list[] = $r;
            if($r->child){
                $new_list = array_merge($new_list,$this->category_table($r->child,$lv + 1));
            }
        }
        return $new_list;

    }

    public function get_category($tax)
    {
        $data_list = get_categories(['taxonomy'=>$tax,'hide_empty'=>false,'pad_counts'=>true]);
        $category = [];
        if($data_list){
            $parent_list = [];
            foreach($data_list as $r){
                if(!isset($parent_list[$r->category_parent])){
                    $parent_list[$r->category_parent] = [];
                }
                $parent_list[$r->category_parent][] = $r;
            }
            foreach($parent_list[0] as $r){
                $child = isset($parent_list[$r->cat_ID]) ? $parent_list[$r->cat_ID] : [];
                if($child){
                    $this->walk_category($child,$parent_list);
                }
                $r->child = $child;
                $category[] = $r;
            }

        }
        return $category;
    }

    public function walk_category($list,$parent_list)
    {
        foreach ($list as $r){
            $child = isset($parent_list[$r->cat_ID]) ? $parent_list[$r->cat_ID] : [];
            if($child){
                $child = $this->walk_category($child,$parent_list);
            }
            $r->child = $child;
        }
        return $list;
    }

    public static function cnf($key=null,$default=null){
        //['switch'=>1,'need_member'=>0,'display_count'=>0,'sticky_mode'=>0,'btn_align'=>0,'remark'=>''];
        static $_option = array();
        if(!$_option){
            $_option = [];
            if(get_option('wb_magicpost_ver',0)){
                $_option = get_option('magicpost_move');
            }
            if(!$_option || !is_array($_option)){
                $_option = [];
            }
            $default_conf = [
                'switch'=>0,
            ];
            foreach ($default_conf as $k=>$v){
                if(!isset($_option[$k]))$_option[$k] = $v;
            }
        }

        if(null === $key){
            return $_option;
        }

        if(isset($_option[$key])){
            return $_option[$key];
        }

        return $default;
    }

    public static function set_active($switch)
    {
        $opt = self::cnf();
        $opt['switch'] = $switch;
        update_option( 'magicpost_move', $opt );
    }

    public static function get_active()
    {
        return self::cnf('switch');
    }

    public function sanitize_text($v,$skip_key = []){

        if(is_array($v))foreach($v as $sk=>$sv){
            if($skip_key && in_array($sk,$skip_key)){
                continue;
            }
            if(is_array($sv)){
                $v[$sk] = $this->sanitize_text($sv,$skip_key);
            }else if(is_string($sv)){
                $v[$sk] = sanitize_text_field($sv);
            }
        }else if(is_string($v)){
            $v = sanitize_text_field($v);
        }
        return $v;
    }

}