<?php
/**
 * Author: wbolt team
 * Author URI: https://www.wbolt.com
 */

class WB_MagicPost_Translate
{
    public $debug = false;
    public $msg = null;

    public function __construct()
    {
        //$cnf = self::cnf();
        if($this->can_use()){
            add_action('translate_single_post',array($this,'translate_single_post'));
            add_action('magic_post_translate_post',array($this,'translate_post_cron'));
            if(!wp_next_scheduled('magic_post_translate_post')){
                $time = strtotime(current_time('Y-m-d H:00:00',1)) + 10;
                wp_schedule_event($time, 'hourly', 'magic_post_translate_post');
            }
        }

        if (is_admin()) {
            add_action('wp_ajax_magicpost',array($this, 'magicpost_ajax'));
            if($this->can_use()){
                add_action('admin_init',array($this, 'admin_init_bulk_actions'));
                add_filter('post_row_actions',array($this,'post_row_actions'),99,2);
            }
            add_action('restrict_manage_posts', array($this,'restrict_manage_posts'), 10, 2);
            add_action('parse_query',array($this,'admin_parse_query'));
        }
    }

    public function admin_parse_query($obj){
        global $wpdb, $current_user;
        $is_trans = isset($_GET['is_trans']) && $_GET['is_trans'] ? intval($_GET['is_trans']) : 0;
        if ($is_trans && $current_user && $current_user->has_cap('edit_posts') && get_option('wb_magicpost_ver',0)) {

            if($is_trans == 1){
                $obj->query_vars['meta_key'] = 'm-p-t-s';
                $obj->query_vars['meta_value'] = '1';
            }else if($is_trans == 2) {
                if(!isset($obj->query_vars['meta_query'])){
                    $obj->query_vars['meta_query'] = array();
                }
                $obj->query_vars['meta_query'][] = array(
                    'relation'=>'OR',
                    array('key'=>'m-p-t-s','compare'=>'NOT EXISTS'),
                    //array('key'=>'url_in_baidu','value'=>'2'),
                );
            }
        }
    }

    public function restrict_manage_posts($post_type, $which){
        if(current_user_can('edit_posts') && get_option('wb_magicpost_ver',0)) {

            if(!self::cnf('switch')){
                return;
            }
            $is_trans = isset($_GET['is_trans']) && $_GET['is_trans'] ? intval($_GET['is_trans']) : 0;
            echo '<select name="is_trans"><option value="">翻译状态</option>';
            foreach (array(1=>'已翻译',2=>'未翻译') as $k=>$v) {
                echo '<option value="' . $k . '" ' . ($is_trans == $k ? 'selected' : '') . '>' . $v . '</option>';
            }
            echo '</select>';

        }
    }

    public function admin_init_bulk_actions()
    {
        global $wp_post_types;

        if($wp_post_types && is_array($wp_post_types))foreach($wp_post_types as $type) {
            if(in_array($type->name,['attachment'])){
                continue;
            }
            if ($type->public) {
                add_filter('bulk_actions-edit-'.$type->name,array($this,'bulk_actions'),90);
            }
        }
    }

    public function post_row_actions($actions, $post)
    {
        static $has_inline_js = false;
        if($this->can_use() && current_user_can('edit_post',$post->ID)){
            do{
                if($post->post_status != 'draft'){
                    break;
                }
                if(empty($post->post_content)){
                    break;
                }
                if(empty($post->post_title)){
                    break;
                }

                $trans_state = get_post_meta($post->ID,'m-p-t-s',1);
                if($trans_state === '1'){
                    break;
                }
                $actions['magicpos_translate_action'] = '<a class="magicpos_translate_action" data-post_id="'.$post->ID.'" href="javascript:;">翻译</a>';
                if(!$has_inline_js){
                    //print_r($post);
                    $has_inline_js = true;
                    $js = array();
                    $js[] = "jQuery('a.magicpos_translate_action').on('click',function(){var obj = jQuery(this);";
                    $js[] = "jQuery.post(ajaxurl,{action:'magicpost','op':'translate_row',post_id:obj.data('post_id')},function(ret){";
                    $js[] = "if(ret){obj.parents('tr').find('.row-title').parent().append(ret); }obj.remove();";
                    $js[] = "});return false;});";

                    wp_add_inline_script('wp-auth-check',implode('',$js));
                }
            }while(0);

        }
        return $actions;
    }

    public function bulk_actions($actions)
    {
        static $has_bulk_inline_js = false;
        if($this->can_use() && current_user_can('administrator')){
            $actions['magicpost_translate'] = '批量翻译';
            if(!$has_bulk_inline_js && get_option('wb_magicpost_ver',0)){
                $has_bulk_inline_js = true;
                $js = array();
                $fun_js = array();
                $fun_js[] = "var ckb = h('.check-column :checkbox:checked');";
                $fun_js[] = "if(ckb.length<1){return false;}";
                $fun_js[] = "var n =1;ckb.each(function(idx,el){";
                $fun_js[] = "var tr = h(el).parents('tr');if(tr.find('a.magicpos_translate_action').length<1)return;";
                $fun_js[] = " n++;tr.find('.row-title').parent().append('<span class=\"mgt-run\"> [翻译中...]</span>');setTimeout(function(){h.post(ajaxurl,{action:'magicpost','op':'translate_row',post_id:h(el).val()},function(ret){";
                $fun_js[] = "  if(ret){var rp = tr.find('.row-title').parent();rp.find('.mgt-run').remove();rp.append(ret);tr.find('a.magicpos_translate_action').remove(); }";
                $fun_js[] = " });},n * 1000);";
                $fun_js[] = "});";
                $js[] = "(function(h){";
                $js[] = "h('#doaction').on('click',function(e){";
                $js[] = "var btn = h(this);var op = btn.prev().val();";
                $js[] = "if(op=='magicpost_translate'){".implode('',$fun_js)."e.preventDefault();return false;}";

                $js[] = "});";
                $js[] = "})(jQuery);";

                wp_add_inline_script('wp-auth-check',implode('',$js));
            }
        }

        return $actions;
    }

    public function translate_post_cron()
    {
        global $wpdb;
        $this->txt('translate post in cron');

        if(!$this->can_use()){
            return;
        }
        $cnf = self::cnf();
        if(!$cnf['auto']){
            return;
        }

        $num = 50;
        $subsql = " AND NOT EXISTS(SELECT m.meta_id FROM $wpdb->postmeta m WHERE m.meta_key='m-p-t-s' AND m.meta_value='1' AND m.post_id=a.ID)";
        $sql = "SELECT a.ID FROM $wpdb->posts a WHERE post_status='draft' AND post_type='post' $subsql AND post_title<>'' AND post_content <>''";
        $this->txt($sql);
        $post_id = $wpdb->get_col($sql." LIMIT ".$num);
        if(!$post_id)return;
        $time = current_time('U',1) + 10;
        foreach($post_id as $id){
            $time += 10;
            wp_schedule_single_event($time,'translate_single_post',array($id));
        }

        $this->txt('translate post finnish');
    }

    public function translate_single_post($post_id)
    {
        $this->txt('translate_single_post:'.$post_id);
        $state = get_post_meta($post_id,'m-p-t-s',true);
        if($state === '1'){
            return;
        }
        $post = get_post( $post_id );
        if ( ! $post ) {
            return;
        }
        if ( 'draft' !== $post->post_status ) {
            return;
        }
        $ret = $this->translate($post);
        if($ret['code']){
            $error_list = get_option('magicpost_translate_error',[]);
            if(!$error_list || !is_array($error_list)){
                $error_list = [];
            }
            $err = '文章：'.$post->post_title.' 翻译失败，错误：'.$ret['desc'];
            array_unshift($error_list,$err);
            $error_list = array_slice($error_list,0,20);
            update_option('magicpost_translate_error',$error_list,false);
        }else{
            update_post_meta($post->ID,'m-p-t-s','1');
        }
        $this->txt('translate_single_post finnish');
    }

    public function translate($post)
    {
        global $wpdb;
        $ret = ['code'=>0,'desc'=>'success'];
        $cnf = self::cnf();
        if(!$this->can_use()){
            $ret['code'] = 1;
            $ret['desc'] = '翻译不可用';
            return $ret;
        }
        $txt = [];
        $fields = [];
        $trans = $this->get_service($cnf);
        if(in_array('post_title',$cnf['trans']) && $post->post_title){
            $fields[] = 'post_title';
            $txt[] = trim($post->post_title);
        }
        if(in_array('post_content',$cnf['trans']) && $post->post_content){
            $fields[] = 'post_content';
            $txt[] = trim($post->post_content);
        }

        $result = $trans->translate($txt);
        if($result === false){
            $ret['code'] = 1;
            $ret['desc'] = $trans->get_error();
            return $ret;
        }
        if(!isset($result['data'])){
            $ret['code'] = 1;
            $ret['desc'] = 'empty data';
            return $ret;
        }
        $data = $result['data'];
        if(!isset($data['translations']) || !is_array($data['translations'])){
            $ret['code'] = 2;
            $ret['desc'] = 'empty data';
            return $ret;
        }

        $translations = $data['translations'];
        //$fields = ['post_title','post_content'];
        $up = [];
        foreach($translations as $k=>$v){
            if(isset($fields[$k])){
                $field = $fields[$k];
                $up[$field] = $v['translatedText'];
            }
        }
        if($up){
            $up['post_ID'] = $post->ID;
            edit_post($up);
            //edit_post($up);
            //$wpdb->update($wpdb->posts,$up,['ID'=>$post->ID]);
        }

        return $ret;
    }

    public function get_service($cnf)
    {
        $obj = new WB_MagicPost_Google_Translate();

        $obj->key = $cnf['google']['key'];
        $target = ['en-zh'=>['en','zh-CN'],'zh-en'=>['zh-CN','en']];
        $obj->target = $target[$cnf['lan']][1];

        return $obj;
    }

    public function txt($msg)
    {
        if(!$this->debug){
            return;
        }
        $msg = is_array($msg)?json_encode($msg,JSON_UNESCAPED_UNICODE):$msg;
        error_log(current_time('mysql') . " $msg \n",3,MAGICPOST_ROOT.'/translate.log');
    }

    public function magicpost_ajax()
    {
        global $wpdb,$wp_taxonomies,$wp_post_types;

        $op = trim(sanitize_text_field($_POST['op'] ?? ''));

        switch ($op) {
            case 'translate_row':
                $ret = array('code'=>0,'desc'=>'success');
                do {
                    if (!current_user_can('manage_options')) {
                        break;
                    }
                    $id = intval($_POST['post_id'] ?? 0);
                    if(!$id){
                        break;
                    }
                    $trans_state = get_post_meta($id,'m-p-t-s',1);
                    if($trans_state === '1'){
                        break;
                    }
                    $post = get_post($id);
                    $ret = $this->translate($post);
                    if($ret['code']){
                        $error_list = get_option('magicpost_translate_error',[]);
                        if(!$error_list || !is_array($error_list)){
                            $error_list = [];
                        }
                        $err = '文章：'.$post->post_title.' 翻译失败，错误：'.$ret['desc'];
                        array_unshift($error_list,$err);
                        $error_list = array_slice($error_list,0,20);
                        update_option('magicpost_translate_error',$error_list,false);
                    }else{
                        update_post_meta($id,'m-p-t-s','1');
                    }
                }while(0);
                //header('content-type:text/json;charset=utf-8;');
                //echo json_encode($ret);
                if($ret['code']){
                    echo ' [翻译失败:'.$ret['desc'].']';
                }else{
                    echo ' [翻译成功]';
                }
                exit();
                break;

            case 'translate_setting':
                $ret = ['code'=>1,'desc'=>'fail'];
                do{
                    if (!current_user_can('manage_options')) {
                        $ret['desc'] = 'deny';
                        break;
                    }
                    $ret['opt'] = self::cnf();
                    $ret['data'] = get_option('magicpost_translate_error',[]);
                    $ret['code'] = 0;
                    $ret['desc'] = 'success';
                }while(0);
                header('content-type:test/json;charset=utf-8;');
                echo json_encode($ret);
                exit();

                break;

            case 'translate_update':
                $ret = ['code'=>1,'desc'=>'fail'];
                do{
                    if (!current_user_can('manage_options')) {
                        $ret['desc'] = 'deny';
                        break;
                    }
                    $old = self::cnf();
                    $opt = $this->sanitize_text($_POST['opt'] ?? []);
                    if(empty($opt) || !is_array($opt)){
                        $ret['desc'] = 'illegal';
                        break;
                    }
                    if($old['auto'] && !$opt['auto']){
                        wp_clear_scheduled_hook('magic_post_translate_post');
                    }
                    update_option( 'magicpost_translate', $opt );

                    $ret['code'] = 0;
                    $ret['desc'] = 'success';
                }while(0);
                header('content-type:test/json;charset=utf-8;');
                echo json_encode($ret);
                exit();
                break;
        }

    }


    public static function cnf($key=null,$default=null){
        //['switch'=>1,'need_member'=>0,'display_count'=>0,'sticky_mode'=>0,'btn_align'=>0,'remark'=>''];
        static $_option = array();
        if(!$_option){
            $_option = [];
            if(get_option('wb_magicpost_ver',0)){
                $_option = get_option('magicpost_translate');
            }
            if(!$_option || !is_array($_option)){
                $_option = [];
            }
            $default_conf = [
                'switch'=>'0',
                'google'=> [
                    'key'=>'',
                ],
                'trans' => [
                    'post_title',
                    'post_content'
                ],
                'auto' => '0',
                'lan' =>'en-zh'
            ];
            foreach ($default_conf as $k=>$v){
                if(!isset($_option[$k]))$_option[$k] = $v;
            }
            foreach(['google','trans'] as $f){
                if(!is_array($_option[$f])){
                    $_option[$f] = $default_conf[$f];
                }
            }
        }

        if(null === $key){
            return $_option;
        }

        if(isset($_option[$key])){
            return $_option[$key];
        }

        return $default;
    }

    public function can_use()
    {
        if(!get_option('wb_magicpost_ver',0))return false;
        $cnf = self::cnf();
        if(!$cnf['switch'])return false;
        if(!$cnf['google']['key'])return false;
        return true;
    }

    public static function set_active($switch)
    {
        $opt = self::cnf();
        $opt['switch'] = $switch;
        update_option( 'magicpost_translate', $opt );
    }

    public static function get_active()
    {
        return self::cnf('switch');
    }

    public function sanitize_text($v,$skip_key = []){

        if(is_array($v))foreach($v as $sk=>$sv){
            if($skip_key && in_array($sk,$skip_key)){
                continue;
            }
            if(is_array($sv)){
                $v[$sk] = $this->sanitize_text($sv,$skip_key);
            }else if(is_string($sv)){
                $v[$sk] = sanitize_text_field($sv);
            }
        }else if(is_string($v)){
            $v = sanitize_text_field($v);
        }
        return $v;
    }

}